import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:sudoku/config/default_game_settings.dart';
import 'package:sudoku/config/default_global_settings.dart';
import 'package:sudoku/cubit/settings_game_cubit.dart';
import 'package:sudoku/cubit/settings_global_cubit.dart';
import 'package:sudoku/ui/painters/parameter_painter.dart';
import 'package:sudoku/ui/widgets/button_delete_saved_game.dart';
import 'package:sudoku/ui/widgets/button_game_start_new.dart';
import 'package:sudoku/ui/widgets/button_resume_saved_game.dart';
import 'package:sudoku/ui/widgets/parameter_image.dart';

class ScreenParameters extends StatelessWidget {
  const ScreenParameters({super.key, required this.canResume});

  final bool canResume;

  final double separatorHeight = 8.0;

  @override
  Widget build(BuildContext context) {
    final List<Widget> lines = [];

    // Game settings
    for (String code in DefaultGameSettings.availableParameters) {
      lines.add(Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: buildParametersLine(
          code: code,
          isGlobal: false,
        ),
      ));

      lines.add(SizedBox(height: separatorHeight));
    }

    lines.add(SizedBox(height: separatorHeight));
    lines.add(Expanded(
      child: canResume ? const ResumeSavedGameButton() : const StartNewGameButton(),
    ));
    lines.add(SizedBox.square(
      dimension: MediaQuery.of(context).size.width / 4,
      child: canResume ? const DeleteSavedGameButton() : const SizedBox.shrink(),
    ));
    lines.add(SizedBox(height: separatorHeight));

    // Global settings
    for (String code in DefaultGlobalSettings.availableParameters) {
      lines.add(
        Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: buildParametersLine(
            code: code,
            isGlobal: true,
          ),
        ),
      );

      lines.add(SizedBox(height: separatorHeight));
    }

    return Column(
      children: lines,
    );
  }

  List<Widget> buildParametersLine({
    required String code,
    required bool isGlobal,
  }) {
    final List<Widget> parameterButtons = [];

    final List<String> availableValues = isGlobal
        ? DefaultGlobalSettings.getAvailableValues(code)
        : DefaultGameSettings.getAvailableValues(code);

    for (String value in availableValues) {
      final Widget parameterButton = BlocBuilder<GameSettingsCubit, GameSettingsState>(
        builder: (BuildContext context, GameSettingsState gameSettingsState) {
          return BlocBuilder<GlobalSettingsCubit, GlobalSettingsState>(
            builder: (BuildContext context, GlobalSettingsState globalSettingsState) {
              final GameSettingsCubit gameSettingsCubit =
                  BlocProvider.of<GameSettingsCubit>(context);
              final GlobalSettingsCubit globalSettingsCubit =
                  BlocProvider.of<GlobalSettingsCubit>(context);

              final String currentValue = isGlobal
                  ? globalSettingsCubit.getParameterValue(code)
                  : gameSettingsCubit.getParameterValue(code);

              final bool isActive = (value == currentValue);

              final double displayWidth = MediaQuery.of(context).size.width;
              final double itemWidth = displayWidth / availableValues.length - 25;

              final bool isPainterOrAsset = (code != DefaultGlobalSettings.parameterCodeSkin);

              return TextButton(
                child: Container(
                  child: isPainterOrAsset
                      ? CustomPaint(
                          size: Size(itemWidth, itemWidth),
                          willChange: false,
                          painter: ParameterPainter(
                            code: code,
                            value: value,
                            isSelected: isActive,
                            gameSettings: gameSettingsState.settings,
                            globalSettings: globalSettingsState.settings,
                          ),
                          isComplex: true,
                        )
                      : SizedBox.square(
                          dimension: itemWidth,
                          child: ParameterImage(
                            code: code,
                            value: value,
                            isSelected: isActive,
                          ),
                        ),
                ),
                onPressed: () => isGlobal
                    ? globalSettingsCubit.setParameterValue(code, value)
                    : gameSettingsCubit.setParameterValue(code, value),
              );
            },
          );
        },
      );

      parameterButtons.add(parameterButton);
    }

    return parameterButtons;
  }
}
