import 'dart:math';

import 'package:sudoku/models/board.dart';
import 'package:sudoku/models/cell.dart';
import 'package:sudoku/models/cell_location.dart';
import 'package:sudoku/models/types.dart';
import 'package:sudoku/utils/tools.dart';

class BoardUtils {
  static BoardCells createEmptyBoard(final int boardSize) {
    final BoardCells cells = [];
    for (int rowIndex = 0; rowIndex < boardSize; rowIndex++) {
      final List<Cell> row = [];
      for (int colIndex = 0; colIndex < boardSize; colIndex++) {
        row.add(Cell(
          location: CellLocation.go(rowIndex, colIndex),
          value: 0,
          isFixed: false,
        ));
      }
      cells.add(row);
    }

    return cells;
  }

  static Board createBoardFromTemplate({
    required String template,
    required bool isSymetric,
  }) {
    BoardCells cells = [];
    final int boardSize = int.parse(pow(template.length, 1 / 2).toStringAsFixed(0));

    const String stringValues = '0123456789ABCDEFG';

    int index = 0;
    for (int rowIndex = 0; rowIndex < boardSize; rowIndex++) {
      final List<Cell> row = [];
      for (int colIndex = 0; colIndex < boardSize; colIndex++) {
        final String stringValue = template[index++];
        final int value = stringValues.indexOf(stringValue);
        row.add(Cell(
          location: CellLocation.go(rowIndex, colIndex),
          value: value,
          isFixed: (value != 0),
        ));
      }
      cells.add(row);
    }

    const List<String> allowedFlip = ['none', 'horizontal', 'vertical'];
    List<String> allowedRotate = ['none', 'left', 'right'];

    // Forbid rotation if blocks are not symetric
    if (!isSymetric) {
      allowedRotate = ['none'];
    }

    final Random rand = Random();
    final String flip = allowedFlip[rand.nextInt(allowedFlip.length)];
    final String rotate = allowedRotate[rand.nextInt(allowedRotate.length)];

    printlog('flip board: $flip');
    printlog('rotate board: $rotate');

    switch (flip) {
      case 'horizontal':
        {
          final BoardCells transformedBoard = createEmptyBoard(boardSize);
          for (int rowIndex = 0; rowIndex < boardSize; rowIndex++) {
            for (int colIndex = 0; colIndex < boardSize; colIndex++) {
              transformedBoard[rowIndex][colIndex] = Cell(
                location: CellLocation.go(rowIndex, colIndex),
                value: cells[boardSize - rowIndex - 1][colIndex].value,
                isFixed: false,
              );
            }
          }
          cells = transformedBoard;
        }
        break;
      case 'vertical':
        {
          final BoardCells transformedBoard = createEmptyBoard(boardSize);
          for (int rowIndex = 0; rowIndex < boardSize; rowIndex++) {
            for (int colIndex = 0; colIndex < boardSize; colIndex++) {
              transformedBoard[rowIndex][colIndex] = Cell(
                location: CellLocation.go(rowIndex, colIndex),
                value: cells[rowIndex][boardSize - colIndex - 1].value,
                isFixed: false,
              );
            }
          }
          cells = transformedBoard;
        }
        break;
    }

    switch (rotate) {
      case 'left':
        {
          final BoardCells transformedBoard = createEmptyBoard(boardSize);
          for (int rowIndex = 0; rowIndex < boardSize; rowIndex++) {
            for (int colIndex = 0; colIndex < boardSize; colIndex++) {
              transformedBoard[rowIndex][colIndex] = Cell(
                location: CellLocation.go(rowIndex, colIndex),
                value: cells[colIndex][boardSize - rowIndex - 1].value,
                isFixed: false,
              );
            }
          }
          cells = transformedBoard;
        }
        break;
      case 'right':
        {
          final BoardCells transformedBoard = createEmptyBoard(boardSize);
          for (int rowIndex = 0; rowIndex < boardSize; rowIndex++) {
            for (int colIndex = 0; colIndex < boardSize; colIndex++) {
              transformedBoard[rowIndex][colIndex] = Cell(
                location: CellLocation.go(rowIndex, colIndex),
                value: cells[boardSize - colIndex - 1][rowIndex].value,
                isFixed: false,
              );
            }
          }
          cells = transformedBoard;
        }
        break;
    }

    // Fix cells fixed states
    for (int rowIndex = 0; rowIndex < boardSize; rowIndex++) {
      for (int colIndex = 0; colIndex < boardSize; colIndex++) {
        cells[rowIndex][colIndex] = Cell(
          location: CellLocation.go(rowIndex, colIndex),
          value: cells[rowIndex][colIndex].value,
          isFixed: (cells[rowIndex][colIndex].value != 0) ? true : false,
        );
      }
    }

    return Board.createNew(
      cells: cells,
    );
  }
}
